﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace SLSimpleMap.UserControls
{
    public partial class WebcamUC : UserControl
    {
        public WebcamUC()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(UserControl_Loaded);
        }

        #region Variables

        private SLSimpleMap.Entities.Webcam webcam = null;
        private ReadOnlyCollection<VideoCaptureDevice> webcams = null;
        private ObservableCollection<WriteableBitmap> capturedImages = new ObservableCollection<WriteableBitmap>();
        //private PixelShader pixelShader = null;

        #endregion

        #region UserControl Loaded

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // Initialisation des Handlers
            this.StartButton.Click += new RoutedEventHandler(StartButton_Click);
            this.StopButton.Click += new RoutedEventHandler(StopButton_Click);
            this.CaptureButton.Click += new RoutedEventHandler(CaptureButton_Click);
            this.VideoPreview.MouseLeftButtonDown += new MouseButtonEventHandler(StopButton_Click);

            // Initialisation des Handlers Effects
            this.EffectSelect.SelectionChanged += new SelectionChangedEventHandler(EffectSelect_SelectionChanged);

            webcam = new SLSimpleMap.Entities.Webcam();
            webcams = CaptureDeviceConfiguration.GetAvailableVideoCaptureDevices();

            if(webcams != null) {
                this.WebcamSelect.DisplayMemberPath = "FriendlyName";
                this.WebcamSelect.ItemsSource = webcams;
                this.ImageStorage.ItemsSource = capturedImages;
                this.WebcamSelect.SelectionChanged += new SelectionChangedEventHandler(WebcamSelect_SelectionChanged);
            }

            if (webcam.VideoDevice == null)
            {
                this.StartButton.IsEnabled = false;
                this.StopButton.IsEnabled = false;
                this.CaptureButton.IsEnabled = false;
            }
        }

        void EffectSelect_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBoxItem content = (ComboBoxItem)this.EffectSelect.SelectedItem;

            switch (content.Content.ToString())
            {
                case "Normal":
                    this.VideoPreview.Effect = null;
                    break;
                case "Invert":
                    this.VideoPreview.Effect = this.Resources["EffectInvert"] as Effect;
                    break;
                case "Pixel":
                    this.VideoPreview.Effect = this.Resources["EffectPixel"] as Effect;
                    break;
                case "Ripple":
                    this.VideoPreview.Effect = this.Resources["EffectRipple"] as Effect;
                    break;
                case "Swirl":
                    this.VideoPreview.Effect = this.Resources["EffectSwirl"] as Effect;
                    break;
                default:
                    this.VideoPreview.Effect = null;
                    break;
            }
        }

        #endregion

        #region Webcam Control

        void WebcamSelect_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            webcam.VideoDevice = (VideoCaptureDevice)this.WebcamSelect.SelectedItem;
            this.StartButton.IsEnabled = true;

            if (webcam.CaptureSource != null && webcam.CaptureSource.State == CaptureState.Started)
            {
                webcam.CaptureSource.Stop();

                if (webcam.CaptureSource != null)
                    this.StartButton.IsEnabled = true;
                else
                    this.StartButton.IsEnabled = false;
                this.StopButton.IsEnabled = false;
                this.CaptureButton.IsEnabled = false;
            }
        }

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {

            if (webcam.CaptureSource != null && webcam.CaptureSource.State == CaptureState.Started)
                webcam.CaptureSource.Stop();

            if (CaptureDeviceConfiguration.AllowedDeviceAccess || CaptureDeviceConfiguration.RequestDeviceAccess())
            {
                this.StartButton.IsEnabled = false;
                this.StopButton.IsEnabled = true;
                this.CaptureButton.IsEnabled = true;

                if (webcam.VideoDevice != null)
                {
                    webcam.CaptureSource = new CaptureSource();
                    webcam.CaptureSource.VideoCaptureDevice = webcam.VideoDevice;

                    try {
                        webcam.CaptureSource.Start();
                    }
                    catch (InvalidCastException e1) {
                        MessageBox.Show("Erreur : "+ e1.Message);
                    }

                    VideoBrush videoBrush = new VideoBrush();
                    videoBrush.Stretch = Stretch.Fill;
                    videoBrush.SetSource(webcam.CaptureSource);

                    webcam.DefaultBrush = this.VideoPreview.Fill;
                    this.VideoPreview.Fill = videoBrush;
                }
                else
                    MessageBox.Show("Erreur : Aucune Webcam n'a été sélectionné");
            }
            else
                MessageBox.Show("Erreur : Accès refusé par l'utilisateur ou aucun périphérique vidéo ou audio n'est disponible.");
        }

        private void StopButton_Click(object sender, RoutedEventArgs e)
        {
            if (webcam.CaptureSource != null && webcam.CaptureSource.State == CaptureState.Started)
            {
                webcam.CaptureSource.Stop();
                this.VideoPreview.Fill = webcam.DefaultBrush;

                this.StartButton.IsEnabled = true;
                this.StopButton.IsEnabled = false;
                this.CaptureButton.IsEnabled = false;
            }
            else
                MessageBox.Show("Erreur : Aucun flux vidéo n'a été démarré.");
        }

        private void CaptureButton_Click(object sender, RoutedEventArgs e)
        {
            System.IO.IsolatedStorage.IsolatedStorageFile isf = System.IO.IsolatedStorage.IsolatedStorageFile.GetUserStoreForApplication();

            var stream = isf.CreateFile("toto.bmp");
            if (isf.Quota < 5000000)
                isf.IncreaseQuotaTo(5000000);
            if (webcam.CaptureSource != null)
            {
                webcam.CaptureSource.CaptureImageCompleted += new EventHandler<CaptureImageCompletedEventArgs>(CaptureSource_CaptureImageCompleted);
                webcam.CaptureSource.CaptureFailed += (s2, e2) =>
                {
                    MessageBox.Show("Capture d'image erroné : " + e2.ErrorException.Message);
                };
                webcam.CaptureSource.CaptureImageAsync();
            }
            else
                MessageBox.Show("Erreur : Le flux vidéo n'a pas été démarré.");
        }

        private void CaptureSource_CaptureImageCompleted(object sender, CaptureImageCompletedEventArgs e)
        {

            capturedImages.Add(e.Result);
            var t = capturedImages.Count;
            var t2 = this.ImageStorage.Items.Count;
            MessageBox.Show(Convert.ToString(t2));

            webcam.CaptureSource.CaptureImageCompleted -= new EventHandler<CaptureImageCompletedEventArgs>(CaptureSource_CaptureImageCompleted);
        }

        #endregion
    }
}
