﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using SLSimpleMap.BingMaps.SearchService;
using Microsoft.Maps.MapControl;

namespace SimpleMap.Views
{
    public partial class MapServiceSearch : Page
    {
        #region Private fields

        SearchServiceClient client = new SearchServiceClient();

        #endregion

        public MapServiceSearch()
        {
            InitializeComponent();

            this.Loaded += new RoutedEventHandler(MapServiceSearch_Loaded);

            // Associate async events
            client.SearchCompleted += new EventHandler<SearchCompletedEventArgs>(client_SearchCompleted);
        }

        void client_SearchCompleted(object sender, SearchCompletedEventArgs e)
        {
            if (e.Result.ResponseSummary.StatusCode == ResponseStatusCode.Success)
            {
                this.map.Children.Clear();
                LocationCollection locs = new LocationCollection();

                foreach (var curResult in e.Result.ResultSets[0].Results)
                {
                    Location location = new Location(curResult.LocationData.Locations[0]);

                    Pushpin pin = new Pushpin();
                    pin.Tag = curResult.Name;
                    pin.Location = location;
                    ToolTipService.SetToolTip(pin, pin.Tag.ToString());

                    this.map.Children.Add(pin);

                    locs.Add(location);
                }

                this.map.SetView(new LocationRect(locs.ToList()));
            }
        }

        void MapServiceSearch_Loaded(object sender, RoutedEventArgs e)
        {
            this.map.MouseClick += new EventHandler<Microsoft.Maps.MapControl.MapMouseEventArgs>(map_MouseClick);
        }

        void map_MouseClick(object sender, Microsoft.Maps.MapControl.MapMouseEventArgs e)
        {
            Location clickedLoc = null;
            this.map.TryViewportPointToLocation(e.ViewportPoint, out clickedLoc);

            if (clickedLoc != null)
            {
                SearchRequest request = new SearchRequest();

                // Set the culture
                request.Culture = this.map.Culture;

                // Set the query
                request.Query = "Pizza";

                // Set the query options
                request.SearchOptions = new SearchOptions();
                request.SearchOptions.Radius = 20; // 20km

                // Set the User profile information
                request.UserProfile = new UserProfile();
                request.UserProfile.CurrentLocation = new UserLocation()
                {
                    Latitude = clickedLoc.Latitude,
                    Longitude = clickedLoc.Longitude
                };
                request.UserProfile.DistanceUnit = DistanceUnit.Kilometer;

                // Suppress faults
                request.ExecutionOptions = new ExecutionOptions();
                request.ExecutionOptions.SuppressFaults = true;

                // Set credentials
                request.Credentials = new Credentials();
                request.Credentials.ApplicationId = "AvQU0yPlMDJzt3ne0iuhmUOTFTDDr_7tTS52IlEGicofbIzOGLP-Lzp0M6INtaH4";

                // Call the WS
                this.client.SearchAsync(request);
            }

        }

        // Executes when the user navigates to this page.
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }
    }
}